use std::process::exit;

use jni::JavaVM;
use jni::JNIVersion;
use jni::InitArgsBuilder;
use jni::JNIEnv;
use jni::objects::JValue;
use jni::objects::JObject;
use jni::sys::jshort;

use std::mem::transmute;

#[no_mangle]
pub unsafe extern "system" fn Java_sib_bedohswe_LabaStack_pop<'local>(mut env: JNIEnv<'local>, object: JObject<'local>) -> jshort {
    let pop_addr = env.get_field(object, "pop_addr", "J").unwrap().j().unwrap();
    let pop = transmute::<i64, fn() -> u16>(pop_addr);
    return pop() as i16;
}

#[no_mangle]
pub unsafe extern "system" fn Java_sib_bedohswe_LabaStack_push<'local>(mut env: JNIEnv<'local>, object: JObject<'local>, number: jshort) {
    let push_addr = env.get_field(object, "push_addr", "J").unwrap().j().unwrap();
    let push = transmute::<i64, fn(u16)>(push_addr);
    push(number as u16);
}

#[no_mangle]
pub unsafe extern "system" fn Java_sib_bedohswe_LabaStack_len<'local>(mut env: JNIEnv<'local>, object: JObject<'local>) -> jshort {
    let len_addr = env.get_field(object, "len_addr", "J").unwrap().j().unwrap();
    let len = transmute::<i64, fn() -> u16>(len_addr);
    return len() as i16;
}

#[no_mangle]
extern "C" fn labashka(pop: extern fn() -> u16, push: extern fn(u16), len: extern fn() -> usize, max: usize) {
    let cnlen = pop() as usize;
    let mut vclassname = vec![0 as u8; cnlen];
    for i in (0..cnlen).rev() {
        vclassname[i] = pop() as u8;
    }
    let classname: String = String::from_utf8(vclassname).unwrap().as_str().replace(".", "/");
    #[cfg(target_family = "windows")]
    let jvm_args = InitArgsBuilder::new()
        .version(JNIVersion::V8)
        .option("-Djava.class.path=./liblbji/;.")
        .option("-Djava.library.path=./liblbji/")
        .build()
        .unwrap();
    #[cfg(not(target_family = "windows"))]
    let jvm_args = InitArgsBuilder::new()
        .version(JNIVersion::V8)
        .option("-Djava.class.path=./liblbji/:.")
        .option("-Djava.library.path=./liblbji/")
        .build()
        .unwrap();
    let jvm = JavaVM::new(jvm_args).unwrap();
    let mut env = jvm.attach_current_thread().unwrap();
    let class = match env.find_class(classname) {
        Ok(r) => r,
        Err(e) => {eprintln!("{}", e); exit(1);}
    };
    let labastackclass = match env.find_class("sib/bedohswe/LabaStack") {
        Ok(r) => r,
        Err(e) => {eprintln!("{}", e); exit(1);}
    };
    let labastackobject = match env.new_object(labastackclass, "(JJJJ)V", &[JValue::from(pop as i64), JValue::from(push as i64), JValue::from(len as i64), JValue::from(max as i64), ]) {
        Ok(r) => r,
        Err(e) => {eprintln!("{}", e); exit(1);}
    };
    if let Err(e) = env.call_static_method(class, "labashka", "(Lsib/bedohswe/LabaStack;)V", &[JValue::from(&labastackobject)]) {
        eprintln!("{}", e);
        exit(1);
    }
}
